import { existsSync, readFileSync, lstatSync } from 'node:fs';
import path from 'node:path';
import { Command, Option, InvalidArgumentError } from 'commander';
import { LogLevel } from '../../types/index.js';
import { LWCServer } from '../../server/index.js';
import ConsoleLogger from '../../server/consoleLogger.js';
export function startDevServer(config) {
    return new LWCServer(config);
}
function getTargetOption() {
    return new Option('-t, --target <target>', '[string] configured target for deployment')
        .choices(['lex', 'mrt', 'mobile'])
        .default('lex');
}
function getPortOption() {
    return new Option('-p, --port <number>', `[number] set port`)
        .env('PORT')
        .argParser(parseNumArg)
        .makeOptionMandatory();
}
function getWorkspaceOption() {
    return new Option('-w, --workspace <workspace>', '[string] configured workspace for development')
        .choices(['internal', 'SalesforceCLI', 'mrt'])
        .default('internal');
}
function parseNumArg(value) {
    const parsedValue = parseInt(value, 10);
    if (isNaN(parsedValue)) {
        throw new InvalidArgumentError('Not a number.');
    }
    return parsedValue;
}
const consoleLogger = new ConsoleLogger(LogLevel.info);
function normalizeConfig({ config, port, target, logLevel, workspace, }) {
    let rootDir = process.cwd();
    const configFile = path.resolve(rootDir, config);
    const modulePaths = [];
    if (!existsSync(configFile) || !lstatSync(configFile).isFile()) {
        consoleLogger.log(LogLevel.error, `Failed to find a valid config file at ${configFile}.`);
        process.exit(1);
    }
    const parsedConfig = JSON.parse(readFileSync(configFile, 'utf-8'));
    consoleLogger.log(LogLevel.debug, `Found config file at "${configFile}" and initializing dev server with it.`);
    rootDir = parsedConfig.rootDir ?? rootDir;
    if (parsedConfig.lwc?.modules) {
        parsedConfig.lwc.modules.forEach((entry) => {
            if (entry.dir) {
                const resolvedDir = entry.dir.replace(/^\$rootDir/, rootDir);
                if (existsSync(resolvedDir)) {
                    modulePaths.push(resolvedDir);
                    consoleLogger.log(LogLevel.debug, `FS-Watcher: Adding ${resolvedDir} to watch.`);
                }
                else {
                    consoleLogger.log(LogLevel.warn, `FS-Watcher: ${resolvedDir} does not exist, ignoring.`);
                }
            }
        });
    }
    const normalizedConfig = {
        rootDir,
        port,
        protocol: 'wss',
        host: 'localhost',
        // Component paths to watch
        paths: modulePaths,
        targets: [target],
        workspace,
        logLevel,
    };
    if (parsedConfig.https) {
        normalizedConfig.https = parseHttpsConfig(parsedConfig.https, path.dirname(configFile));
    }
    return normalizedConfig;
}
function parseHttpsConfig(config, configDir) {
    if (typeof config !== 'object')
        throw new Error('Expected config to be an object');
    if (config === null)
        throw new Error(`config can't be null`);
    const result = {};
    // could solve this repetition, but I don't know how to resolve typescript
    if ('cert' in config && typeof config.cert === 'string') {
        result.cert = readFileSync(path.resolve(configDir, config.cert), 'utf-8');
    }
    if ('key' in config && typeof config.key === 'string') {
        result.key = readFileSync(path.resolve(configDir, config.key), 'utf-8');
    }
    if ('pfx' in config && typeof config.pfx === 'string') {
        result.pfx = readFileSync(path.resolve(configDir, config.pfx));
    }
    if ('passphrase' in config && typeof config.passphrase === 'string') {
        result.passphrase = config.passphrase;
    }
    return result;
}
export function createDevServer() {
    return new Command('start')
        .aliases(['preview'])
        .description('Start Dev Server')
        .addOption(getPortOption())
        .addOption(getTargetOption())
        .addOption(getWorkspaceOption())
        .action((_options, cmd) => {
        const { config, port, target, logLevel, workspace } = cmd.optsWithGlobals();
        const serverConfig = normalizeConfig({
            config,
            port,
            target,
            logLevel: LogLevel[logLevel],
            workspace,
        });
        startDevServer(serverConfig);
    });
}
//# sourceMappingURL=devServer.js.map